#!/bin/sh
#
# Copyright (c) 1998 Universities Research Association, Inc.
#       All Rights Reserved
#
#
# This material resulted from work developed under a Government Contract and
# is subject to the following license:  The Government retains a paid-up,
# nonexclusive, irrevocable worldwide license to reproduce, prepare derivative
# works, perform publicly and display publicly by or for the Government,
# including the right to distribute to other Government contractors.  Neither
# the United States nor the United States Department of Energy, nor any of
# their employees, makes any warranty, express or implied, or assumes any
# legal liability or responsibility for the accuracy, completeness, or
# usefulness of any information, apparatus, product, or process disclosed, or
# represents that its use would not infringe privately owned rights.
#
# The setups.sh script is designed to be sourced at shell invocation
# including the time of login.  It uses several other scripts located in  
# ~products/etc to determine the 
# PRODUCTS and UPS_DIR environment variables.This script and all the 
# scripts utilized are created when a version of ups is configured.

# This script is sourced by the fermi.sh files to bootstrap the
# ups working environment and aliases.
# If you need to reestablish missing aliases in a subshell process,
# use the following code:
#    . `${UPS_DIR}/bin/ups setup ups`
# 
# NOTE: The '.' in the preceeding line is significant.
#
# The scripts utilized by the setups.sh script are:
# 
#    ~products/etc/setups_products_init.sh
#
#    ~products/etc/setups.common,
#    as a last resort.
#
#    ~products/etc/setups.cache.'<flavor>' 
#    is used to determine the value of UPS_DIR.
#    
#
# ===		SYS ADMIN AREA		=====
#
# The following shell variables are defined here:
#
#       Setups_Home
#
#       Name_Of_Ups
#
# Please check their values and change, if necessary, following
# their descriptions below.
#
#
# Because the shell languages manage to lose the filename
# of the file being sourced the Setups_Home variable
# exists to assist in installing the codes on non Fermilab
# machines.  Change this to the directory in which the
# setups.* scripts live.

Setups_Home=~products/etc

# Name_Of_Ups should always be ups at Fermilab, unless in beta test

Name_Of_Ups=ups

BootStrap_Dir=/fnal/ups/db/.upsfiles/configure/v4_3_Linux+2_
  
if test -x /bin/echo
then
    ECHO=/bin/echo
elif test  -x /usr/bin/echo
then
    ECHO=/usr/bin/echo
else
    ECHO=echo
fi

# === DO NOT EDIT BELOW THIS LINE === DO NOT REMOVE THIS LINE ===


if test `echo $Setups_Home|cut -c1` != '/'
then
    TeMp=/tmp
    if test x${TMPDIR:-} != x
    then
        TeMp=$TMPDIR
    fi
    for sh in bash ksh ash sh
    do
	if test -x /usr/bin/${sh}
	then
	    Name_Of_Sh=/usr/bin/${sh}
	    break
	elif test -x /bin/${sh}
	then
	    Name_Of_Sh=/bin/${sh}
	    break
	fi
    done
    cat <<XxEoF >$TeMp/Ash_$$
#!${Name_Of_Sh}
eval dir=\$1
$ECHO \$dir
XxEoF
    chmod +x $TeMp/Ash_$$
    Setups_Home=`(unset ENV; $TeMp/Ash_$$ $Setups_Home )`
    rm -f $TeMp/Ash_$$ 
fi

export Setups_Home

Setups_Current=1

if test $Setups_Current = 0
then
    Setups_Home=$BootStrap_Dir
fi

SETUPS_DIR=$Setups_Home; export SETUPS_DIR

UPS_SHELL=sh; export UPS_SHELL
#
# Determine probable flavor names for the version of ups
# which would execute on this platform.
#
Node=`uname -n|cut -f1 -d'.'`
MACH_OS=`uname -s`
if test `echo $MACH_OS|cut -c1-4` = IRIX
then
    MACH_OS=IRIX
fi
export MACH_OS
Flavor=$MACH_OS'+'`uname -r|cut -f1 -d'.'`
if test $MACH_OS = AIX
then
    Flavor=$MACH_OS'+'`uname -v`
fi
if test $MACH_OS = HP-UX
then
    Flavor=$MACH_OS'+'`uname -r|cut -f2 -d'.'`
fi
if test x`echo ${UPS_DIR:-}|cut -c1` = x/
then
    if test ! -d $UPS_DIR
    then
	unset UPS_DIR
    fi
else
	unset UPS_DIR
fi
#
# Attempt to define the PRODUCTS variable.
#
if test x`echo ${PRODUCTS:-}|cut -c1` != x/
then
    if test -x ${Setups_Home}/setups_products_init.sh 
    then
        eval `(unset ENV;${Setups_Home}/setups_products_init.sh)`
    fi   
fi
#
# Attempt to determine the file name of the cached ups information from the 
# naming convention and the probable flavors which might execute correctly
# on the platform executing this script.  If found, utilize it to define
# a UPS_DIR.
#
if test x`echo ${PRODUCTS:-}|cut -c1` = x/ -a x$UPS_DIR = x
then
    for fl in $Node $Flavor $MACH_OS NULL
    do
        if test -r $Setups_Home/set${Name_Of_Ups}.cache.${fl}
        then
            UPS_DIR=`cat $Setups_Home/set${Name_Of_Ups}.cache.${fl}|grep -v '^#'`
        fi
        if test x${UPS_DIR:-} != x
        then
            if test -d $UPS_DIR
            then
                break
            else
                unset UPS_DIR
            fi
        fi
   done
fi
#
# If we have been unsuccessful so far, search the possible products 
# databases for some ups which might execute on this platform.
#
# This process, although thorough, is very time consuming.
#
if test x${UPS_DIR:-} = x
then
    if test -x $Setups_Home/setups.common
    then
        eval `(unset ENV;$Setups_Home/setups.common sh)`
    fi
fi
if test x${UPS_DIR:-} != x
then
#
# If we have successfully managed to find a usable version of ups,
# give this version of ups the information  found so far and let it 
# determine the BEST version.
#
# This extra step ensures that the appropriate ups search order is
# followed to arive at the final result.  It also is the step executed even
# when PRODUCTS and UPS_DIR are known at entry.  The ups setup script
# may define functions which are NOT inherited by sub-shells.
#
	. `$UPS_DIR/bin/ups setup $Name_Of_Ups`
else
#
# If everything failed, report same and return.
#
# DO NOT EXIT - this might be the login shell.
#
	echo '****'
	echo '**** Unable to initialize UPSII environment'
	echo '****'
fi
#
# Because this is designed to be "sourced", its nice to unset the shell
# variables.
#
unset Node
unset BootStrap_Dir
unset Setups_Current
unset fl
unset Flavor
unset Setups_Home
unset BootStrap_Dir
unset Name_Of_Ups
