#!/bin/csh
#
# Copyright (c) 1998 Universities Research Association, Inc.
#       All Rights Reserved
#
#
# This material resulted from work developed under a Government Contract and
# is subject to the following license:  The Government retains a paid-up,
# nonexclusive, irrevocable worldwide license to reproduce, prepare derivative
# works, perform publicly and display publicly by or for the Government,
# including the right to distribute to other Government contractors.  Neither
# the United States nor the United States Department of Energy, nor any of
# their employees, makes any warranty, express or implied, or assumes any
# legal liability or responsibility for the accuracy, completeness, or
# usefulness of any information, apparatus, product, or process disclosed, or
# represents that its use would not infringe privately owned rights.
#
# The setups.csh script is designed to be sourced at shell invocation
# including the time of login.  It uses several other scripts located
# in ~products/etc to determine the 
# PRODUCTS and UPS_DIR environment variables. 

# This script is sourced by the fermi.csh files to bootstrap the
# ups working environment and aliases.
# If you need to reestablish missing aliases in a subshell process,
# use the following code:
#    source `${UPS_DIR}/bin/ups setup ups`
# 
# This script and all the scripts utilized are created when a version 
# of ups is configured.
#
# The scripts utilized by the setups.csh script are:
# 
#    ~products/etc/setups_products_init.sh
#
#    ~products/etc/setups.common,
#     as a last resort.
#
#    ~products/etc/setups.cache.'<flavor>' 
#    is used to determine the UPS_DIR variable.
#

set Setups_Home=~products/etc

set BootStrap_Dir=/fnal/ups/db/.upsfiles/configure/v4_3_Linux+2_

# =====  DO NOT EDIT BELOW THIS LINE  ===  DO NOT REMOVE THIS LINE =====


set Setups_Current=1

if ( $Setups_Current == 0 ) then
    set Setups_Home=$BootStrap_Dir
endif

setenv SETUPS_DIR $Setups_Home

setenv Setups_Home $Setups_Home

set Name_Of_Ups=ups
setenv UPS_SHELL csh
#
# Determine probable flavor names for the version of ups
# which would execute on this platform.
#
setenv MACH_OS `uname -s`
set mach=`echo $MACH_OS|cut -c1-4`
if ( $mach == IRIX ) setenv MACH_OS IRIX
set Flavor=$MACH_OS'+'`uname -r|cut -f1 -d'.'`
if ( $MACH_OS == AIX ) set Flavor=$MACH_OS'+'`uname -v`
if ( $MACH_OS == HP-UX ) set Flavor=$MACH_OS'+'`uname -r|cut -f2 -d'.'`
#
# Define the PRODUCTS variable, if needed
#
if (  ! $?PRODUCTS ) then
    if ( -x ${Setups_Home}/setups_products_init.sh ) then
        eval `${Setups_Home}/setups_products_init.sh`
    endif
endif
if ( $?UPS_DIR ) then
    if ( x`echo $UPS_DIR|cut -c1` != x/ ) then
        unsetenv UPS_DIR
        unset UPS_DIR
    endif
else
    unsetenv UPS_DIR
    unset UPS_DIR
endif
#
# Attempt to determine the file name of the cached ups information from the 
# naming convention and the probable flavors which might execute correctly
# on the platform executing this script.  If found, utilize it to define
# a usable UPS_DIR.
#
if ( $?PRODUCTS && ! $?UPS_DIR ) then
    foreach fl ( $Flavor $MACH_OS NULL )
        if ( -r ${Setups_Home}/set${Name_Of_Ups}.cache.${fl} ) then
	    setenv UPS_DIR `cat $Setups_Home/set${Name_Of_Ups}.cache.${fl}|grep -v '^#'`
            if ( $?UPS_DIR ) then
               if ( -d $UPS_DIR ) then
	             break
                else
                    unsetenv UPS_DIR
                endif
            endif
        endif
    end
endif
#
# If we have been unsuccessful so far, search the possible products 
# databases for some ups which might execute on the this platform.
#
# This process, although thorough, is very time consuming.
#
if ( ! $?UPS_DIR ) then
    if ( -x $Setups_Home/setups.common ) then
	eval `$Setups_Home/setups.common`
    endif
endif
#
# If everything failed, report same and return.
#
# DO NOT EXIT - this might be the login shell.
#
if (! $?UPS_DIR ) then
    echo '****'
    echo '**** Unable to initialize the UPSII environment'
    echo '****'
else
#
# If we have successfully managed to find a usable version of ups,
# give this version of ups the information  found so far and let it 
# determine the BEST version.
#
# This extra step ensures that the appropriate ups search order
# is followed to arive at the final result.  It also is the step executed
# even when PRODUCTS and UPS_DIR are known at entry.  The ups 
# setup script may set aliases which are NOT inherited by sub-shells.
#
#
    source `$UPS_DIR/bin/ups setup $Name_Of_Ups`
endif
#
# Because this is designed to be sourced, it is nice to unset the local
# shell variables
#
unset NODE
unset node
unset BootStrap_Dir
unset Setups_Current

unset mach
unset fl
unset Flavor
unset Setups_Home
unset Name_Of_Ups
unsetenv Setups_Home
