#!/usr/bin/env perl
#!/usr/local/bin/perl
##########################################################################
# $Id: ups.cgi,v 1.24 1998/07/17 22:50:47 mengel Exp $
#
# FILE:
#       ups.cgi
# 
# DESCRIPTION: 
#       It will parse and execute ups commands.
#
#       The upd client expect any lines printed after UPSR_RETURN_CODE='code' 
#       to originate from STDERR.
#
# AUTHORS:
#       Marc Mengel
#       Lars Rasmussen
#       Margaret Votava
#       Don Walsh
#
#       Fermilab Computing Division
#       Batavia, Il 60510, U.S.A.                                              
#
# MODIFICATIONS:
#       19-Dec-1997, lr, first
#       05-Jan-1998, lr, will now check if request is supported
#
##########################################################################
use File::Basename;

my $prog = 'upsr/$Revision: 1.24 $ ';

# valid commands goes here

my $cmd_base = "/bin/true";
my @ok_cmds = ();

# pick behavior based on $0, default to nothing

if ($0 =~ m|ups.cgi$|) {
    $cmd_base = "$ENV{UPS_DIR}/bin/ups";
    @ok_cmds = 
    ( 
      'hi',
      'env',
      'list',
      'depend',
      'get',
      'exist',
      'flavor',
      'verify'
    );
}
if ($0 =~ m|ups-decl.cgi$|) {
    $cmd_base = "$ENV{UPS_DIR}/bin/ups";
    @ok_cmds = 
    (
      'hi',
      'env',
      'list',
      'depend',
      'get',
      'exist',
      'flavor',
      'touch',
      'declare',
      'undeclare',
    );
}
if ($0 =~ m|upd.cgi$|) {
    $cmd_base = "$ENV{UPD_DIR}/bin/upd";
    @ok_cmds = 
    (
      'hi',
      'env',
      'move_archive_file',
      'move_table_file',
      'move_ups_dir',
    );
}

# check (in a cheap way ?) to see if we was called from 
# a browser or from a command line, and parse arguments.

my $cmd = '';
if ( defined ($cmd = $ENV{QUERY_STRING}) ) {
  $cmd =~ s/&/ /g;
  $cmd =~ s/%([0-9a-f][0-9a-f])/chr(hex($1))/eg;
}
else {
  $cmd = join ' ', @ARGV;
}

if (defined ($ENV{REMOTE_USER})) {
   $ENV{USER} = $ENV{REMOTE_USER};
}

# tell client content type

print "Content-type: text/plain\n\n";

# check if command is supported

$_ = $cmd;

if ( ! check_cmd() ) {
  print "UPSR_RETURN_CODE=-1\n";
  print "$prog say: ERROR: unknown or invalid command '$_'\n";
}

# just say hi

elsif ( /^hi/ ) {
  print "$prog say:\n";
  print '$Id: ups.cgi,v 1.24 1998/07/17 22:50:47 mengel Exp $ ', "\n";
  print_usage();
  print "UPSR_RETURN_CODE=0\n";
}

# print environment

elsif ( /^env/ ) {
  print "$prog say:\n";
  print '$Id: ups.cgi,v 1.24 1998/07/17 22:50:47 mengel Exp $ ', "\n";
  print_env();
  print "UPSR_RETURN_CODE=0\n";
}

# execute ups command

else {

  $err_file = "$$.ups_err";

  # create file name for stderr, if it exist remove it

  if ( defined $ENV{TMPDIR} ) {
    $err_file = "$ENV{TMPDIR}/$err_file";
  }
  else {
    $err_file = "/tmp/$err_file";
  }
  unlink $err_file if ( -e $err_file );

  # build ups command

  my $cmd_str = "${cmd_base} $_";
  print "$prog say: $cmd_str\n";

  # redirect stderr

  open SAVEERR, \*STDERR;    
  if ( open STDERR, ">$err_file" ) {

    # execute ups command

    print `$cmd_str`;    

    # send stderr

    print "UPSR_RETURN_CODE=$?\n";
    if ( open EF, $err_file ) {
      print while <EF>;
      close EF;
    }    
  }
  else {

    print "UPSR_RETURN_CODE=$!\n";
    print "$prog say: ERROR: could not open temp file '$err_file': $!\n";
  }

  # restore STDERR

  unlink $err_file;       
  close STDERR;
  open STDERR, \*SAVEERR;
}

#-------------------------------------------------------------------------
# This one, will just check if command is supported.
#
sub check_cmd {
  foreach $cmd ( @ok_cmds ) {
    return 1 if /^$cmd/;
  }
  return 0;
}

#-------------------------------------------------------------------------
# This one, will just print local environment and include path.
#
sub print_env {
  print "<Environment>\n";
  foreach $_ (sort keys %ENV ) {
    print "$_ = $ENV{$_}\n";
  }
  print "<Include path>\n";
  foreach $_ (@INC) {
    print "$_\n";
  }
}

#-------------------------------------------------------------------------
# This one, will just print Usage
#
sub print_usage {

  my $commands = '';
  foreach ( @ok_cmds ) {
    $commands .= "$_|";
  }
  chop $commands;
  print "Usage: ", basename( $0 ), " <$commands> [ups arguments]\n";
}

#-------------------------------------------------------------------------
# Begin, load and check environment variables.
#
BEGIN {

  # some environment variables to check

  my @env_arr = (
		 'PRODUCTS', 
		 'UPS_DIR', 
		 'UPD_DIR'
		 );

  # will try to load the user specified environment, if file does 
  # not exist we will assume they are defined by other means
  # (e.g in httpd.conf).

  if ( defined( $ENV{UPD_USERCODE} ) ) {
    require "$ENV{UPD_USERCODE}/upsr-init.pl"
	if ( -f "$ENV{UPD_USERCODE}/upsr-init.pl" );
  }

  # check environment

  my @err_arr = ();
  foreach ( @env_arr ) {
    push @err_arr, "ERROR: Environment variable '$_' is not defined.\n"
	unless ( defined $ENV{$_} );
  }
  if ( @err_arr ) {
    print "Content-type: text/plain\n\n";
    print "UPSR_RETURN_CODE=-1\n";
    foreach ( @err_arr ) {
      print $_;
    }
    exit;
  }

  # add USER and SHELL if not defined

  $ENV{USER} = 'httpd' unless ( defined $ENV{USER} );
  $ENV{SHELL} = '/bin/sh' unless ( defined $ENV{SHELL} );

}

1;
