##########################################################################
# $Id: updusr.base.pm,v 1.1 1998/04/22 19:38:33 mengel Exp $
#
# DESCRIPTION: 
#       This module contain routines that are user callouts to get various
#       pieces of information. Users are required to edit this file
#	and put in in $UPD_USERCODE or their home directory. Please
#	look at the comments for each subroutine to determine the 
#	pieces of information that need to be returned.
#
#	This file just provides examples, do NOT use as is. You MUST 
#	read and understand each function and modify it to conform to 
#	the configuration for your system. 
#
# MODIFICATIONS:
#
##########################################################################
package updusr;

use upderr;
use updups;
use updgbl;
use strict;
use integer;
use Exporter();
use vars qw( @ISA @EXPORT );
@ISA = qw( Exporter );

# export this !
@EXPORT = qw (
 	updusr_get_db
 	updusr_get_proddir
	updusr_predeclare
	updusr_postdeclare
	updusr_get_tablefile
	updusr_get_upsdir
	);

##########################################################################
# Exported functions
#


#-------------------------------------------------------------------------
# status = updusr_get_db ( \%inst_spec, \%top_inst_spec, $flags)
#
# This routine must fill in $inst_spec->{database}. It is the path name
# of the ups database into which the given product should be declared.
# Examples are to pick the first database in $PRODUCTS in which this product
# was declared. If it's a new product, then to put it in the first in the
# list.
#
# Input : hash,   inst_spec	instance to get db for
#         hash,   top_inst_spec	for overlaid products, this is the main
#                                 product, otherwise it doesn't matter.
#         scalar, flags 	-O option from addproduct command line
# Output: none
# Return: scalar, status
#
sub updusr_get_db {
    my ($inst_spec, $top_inst_spec,$flags) = @_;
    my @products;			# $PRODUCTS
    my $db;                             # single db

    # put databases into an array
    # ---------------------------
    @products = split (/[ :]/,$ENV{PRODUCTS});

    # Here's an example of how to take the first one from $PRODUCTS
    # -------------------------------------------------------------
    #$inst_spec->{database} = $products[0];

    # Here's an example of how to take from that first db this product
    # is declared in. If not declared, pick first db
    # -------------------------------------------------------------
    foreach $db (@products)             # look throgh all db
      {
      if ( -e "$db/$inst_spec->{product}" )     # found product
         {
         $inst_spec->{database} = $db;      # set db
         last;                  # done
         }
       }
    $inst_spec->{database} = $products[0]       # set to first
        unless ( defined $inst_spec->{database} );    # if no db found

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_get_proddir( \%real_inst_spec, \%top_inst_spec, $flags) 
#
# This routine must fill in $real_inst_spec->{prod_dir} and 
# $real_inst_spec->{unwind_prod_dir}. unwind_prod_dir is the directory
# into which the distribution product is unwound. prod_dir is the
# home directory for the product that is used in the declare command.
# For most installations, these are the same. They might be different
# for products in AFS space and products on NT machines (because
# the ftp in perl can't see paths in cygwin format).
#
# Input : hash,   real_inst_spec  instance to get dir for
#         hash,   top_inst_spec   for overlaid products, this is the main
#                                   product, otherwise it doesn't matter.
#         scalar, flags 	  -O option from addproduct command line
# Output: none
# Return: scalar, status
#
sub updusr_get_proddir {
    my ($real_inst_spec, $top_inst_spec,$flags) = @_;
    my $ods_flavor;                     # flavor following ods convention
    my $inst_spec;                      # instance spec used

    # Are we an overlaid product?
    # If so, we want the information from the overlay
    # -----------------------------------------------
    if ( defined $real_inst_spec->{'_upd_overlay'} &&
         $real_inst_spec->{'_upd_overlay'} ne '' &&
         $real_inst_spec->{'_upd_overlay'} ne '""' )    # we are overlaid
       { $inst_spec = $top_inst_spec; }
    else
       { $inst_spec = $real_inst_spec; }

    # this is what ods wants - /usr/products/<flavor>/<product>/<version>
    # but <flavor> is just IRIX, not IRIX+6 so we need to trim that down.
    # also, directory is "cluster_disk" for vxworks and null products.
    # -------------------------------------------------------------------
    return 0 unless updups_flavor (0,"-H $inst_spec->{flavor} -1",\$ods_flavor);
    $ods_flavor = "cluster_disk" if ($ods_flavor eq "NULL" || 
                                     $ods_flavor eq "VxWorks"); 
    $real_inst_spec->{unwind_prod_dir} = 
       "/usr/products/$ods_flavor/$inst_spec->{product}/$inst_spec->{version}";

    # this is what fnalu wants - /afs/fnal.gov/products/<flavor>/<product>/<version>
    # -------------------------------------------------------------------
    # $real_inst_spec->{unwind_prod_dir} = "/afs/fnal.gov/products/".
    #   "$inst_spec->{flavor}/$inst_spec->{product}/$inst_spec->{version}";

    #=======================================================================
    # get declare dir too
    #=======================================================================
    # for most installations, the declare dir is the same as the unwind dir
    # ---------------------------------------------------------------------
    $real_inst_spec->{prod_dir} = $real_inst_spec->{unwind_prod_dir};

    # for afs installations, the declare dir is different
    # ---------------------------------------------------

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_predeclare( $inst_spec, $flags )
#
# Release a volume (if using the afs file system)
#
# Input : scalar, dir       directory to release
#         scalar, flags     -O option from addproduct command line
# Output: none
# Return: scalar, status
#
sub updusr_predeclare {
    my ($inst_spec,$flags) = @_;

    my $dir = $inst_spec->{unwind_prod_dir};
    # for most installations, this does nothing
    # -----------------------------------------
    return 1;

    # for afs installations, this releases the afs volume
    # ---------------------------------------------------
    #if (0 != system("/usr/local/bin/upd_volrelease $dir")) 
    #   {
    #   &upderr_log(0,"release volume of $dir failed: $!\n");
    #   return 0;
    #   }

    return 1;
}

#-------------------------------------------------------------------------
# status = updusr_postdeclare( $inst_spec, $flags )
#
# Release a volume (if using the afs file system)
#
# Input : scalar, dir       directory to release
#         scalar, flags     -O option from addproduct command line
# Output: none
# Return: scalar, status
#
sub updusr_postdeclare {
    my ($inst_spec,$flags) = @_;

    my $dir = $inst_spec->{};
    # for most installations, this does nothing
    # -----------------------------------------
    return 1;

    # for afs installations, this releases the afs volume
    # ---------------------------------------------------
    #if (0 != system("/usr/local/bin/upd_volrelease $dir")) 
    #   {
    #   &upderr_log(0,"release volume of $dir failed: $!\n");
    #   return 0;
    #   }

    return 1;
}

#-------------------------------------------------------------------------
# status = updusr_get_tablefile ( \%inst_spec, $flags)
#
# get table file location of product instance
#
# This routine must fill in $inst_spec->{table_file), $inst_spec->{table_dir},
# and $inst_spec->{unwind_table_dir}. table_file is the table file name.
# unwind_table_dir is the directory into which the table file is unwound.
# while table_dir is the directory for the table file that is used in the
# ups declare command. For most installations, the table_dir is not need
# in the declare command because it will be in one of the default locations
# that ups looks for. In these cases, it should not be set at all.
#
# Input : hash,   inst_spec   	instance spec
#         scalar, flags 	-O option from addproduct command line
# Output: none
# Return: scalar, status
#
sub updusr_get_tablefile {
    my ($inst_spec, $flags) = @_;

   # table files will typically go in either the ups subdirectory
   # or the ups database
   # ----------------------------------------------------------

    $inst_spec->{unwind_table_dir} = "$inst_spec->{database}/$inst_spec->{product}";

#    $inst_spec->{unwind_table_dir} = "$inst_spec->{unwind_prod_dir}/ups";
    if ($flags =~ m/separate_tables/) {
        $inst_spec->{table_file} = 
            "$inst_spec->{version}$inst_spec->{flavor}.table";
    } else {
        $inst_spec->{table_file} = 
            "$inst_spec->{version}.table";
    }
#   $inst_spec->{table_dir} = "ups";

    $inst_spec->{unwind_table_file} = $inst_spec->{table_file};

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_get_upsdir( \%inst_spec, $flags)
#
# This routine must fill in $inst_spec->{ups_dir} and 
# $inst_spec->{unwind_ups_dir}. unwind_ups_dir is the directory
# into which the ups subdirectory is unwound. ups_dir is the
# home directory for the product that is used in the declare command.
#
# Input : hash,   inst_spec   
#         scalar, flags 	-O option from addproduct command line
# Output: none
# Return: scalar, status
#
sub updusr_get_upsdir {
    my ($inst_spec,$flags) = @_;

    # for most installations, the ups directory falls underneath
    # the product root.
    # ----------------------------------------------------------
    $inst_spec->{ups_dir} = "ups";
    $inst_spec->{unwind_ups_dir} = "$inst_spec->{unwind_prod_dir}/ups";

    return 1;
}

#
#-------------------------------------------------------------------------
# status = updusr_unwind_all ( \%inst_spec, $flags)
#
# Only used on upd server machines
#
# Input : hash,   inst_spec   
#         scalar, flags 	-O option from addproduct command line
# Output: none
# Return: boolean
#
# return true if product should be completely unwound, 0 if partialy
#
sub updusr_unwind_all {
    my ($inst_spec, $flags) = @_;

    upderr_log(3, "updusr_unwind_all( $inst_spec, $flags )\n");
    if ($inst_spec->{flavor} =~ /WIN|NT/) {
        return 1;
    } else {
        return 0;
    }
}

1;

__END__

