
##########################################################################
#
# FILE:
#       updusr.pm
# 
# DESCRIPTION: 
#       This module contain routines that are user callouts to get various
#       pieces of information.
#       (for more info, see pod at the end).
#       This is a sample of how I think a cdf Run1 style layout would
#       be done
#
# MODIFICATIONS:
#	1998-03-27 mengel@fnal.gov	Initial "cdf run1 style" version
#
##########################################################################
package updusr;

use upderr;
use updups;
use updgbl;
use upduti;
use upderr;
use strict;
use integer;
use POSIX;
use File::Path;
use Exporter();
use vars qw( @ISA @EXPORT );
@ISA = qw( Exporter );

# export this !
@EXPORT = qw (
 	updusr_get_db
 	updusr_get_proddir
	updusr_predeclare
	updusr_postdeclare
	updusr_get_tablefile
	updusr_get_upsdir
	updusr_unwind_all
	);

##########################################################################
# Exported functions
#

# Location of kits area
#
$updusr::kitsarea="/scratch2";


#-------------------------------------------------------------------------
# status = updusr_get_db ( \%inst_spec, \%top_inst_spec)
#
# Gets and stores (into my_instance_spec) the database into which the
# product should be declared.
#
# Input : hash, inst_spec	instance to get db for
#         hash, top_inst_spec	for overlaid products, this is the main
#                               product, otherwise it doesnt matter.
#         scalar, dist_db	distribution database
# Output: none
# Return: scalar, status
#
sub updusr_get_db {
    my ($inst_spec, $top_inst_spec, $flags) = @_;
    my @products;			# $PRODUCTS
    my $db;				# single db
    my $dist_node = $g_flags{opt_h};	# distribution node in case we need it.

    if ($flags =~ m/cdfsoft/) {
	$inst_spec->{database} = $ENV{'BFDIST'}/ups_database/declared;
    } else {
	$inst_spec->{database} = /usr/products/ups_database/declared;
    }

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_get_proddir( \%real_inst_spec, \%top_inst_spec) 
#
# Gets and stores (into my_instance_spec) the directory into which the
# product files should be unwound and declared (these might be separated).
#
# Input : hash, real_inst_spec	instance to get dir for
#         hash, top_inst_spec	for overlaid products, this is the main
#                               product, otherwise it doesnt matter.
# Output: none
# Return: scalar, status
#
sub updusr_get_proddir {
    my ($real_inst_spec, $top_inst_spec, $flags) = @_;
    my $inst_spec;			# instance spec used
    my $dist_node = $g_flags{opt_h};	# distribution node in case we need it

    if ($flags =~ m/cdfsoft/ ) {
        my $prod_part;

	if ($inst_spec->{product} eq 'cdfsoft2') {
		$prod_part = 'releases';
	} else {
		$prod_part = $inst_spec->{product};
	}

        # declare it as ~cdfsoft

        $real_inst_spec->{prod_dir} = 
		    '${BFDIST}/packages/' .
		    $prod_part . 
		    "/" .
		    $inst_spec->{version};

        # we think we know ~cdfsoft/dist == $ENV{BFDIST}...
        # so thats the real pathname we'll use

        $real_inst_spec->{unwind_prod_dir} = 
		    $ENV{BFDIST} .  '/packages/' .
		    $prod_part . 
		    "/" .
		    $inst_spec->{version};

    } else {
        $real_inst_spec->{unwind_prod_dir} = 
		    "/usr/products/" . 
		    $inst_spec->{product} . 
		    "/" .
		    $inst_spec->{version};
        $real_inst_spec->{prod_dir} = $real_inst_spec->{unwind_prod_dir};
    }

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_predeclare( $inst_spec )
#
# Release a volume (if using the afs file system)
#
# Input : scalar, dir		directory to release
# Output: none
# Return: scalar, status
#
sub updusr_predeclare {
    my ($dir,$flags) = @_;

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_postdeclare( $inst_spec )
#
# callout after declare
#
# Input : scalar, dir		directory to release
# Output: none
# Return: scalar, status
#
sub updusr_postdeclare {
    my ($dir,$flags) = @_;

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_get_tablefile ( \%inst_spec)
#
# get table file location of product instance
#
# Input : hash, inst_spec	
# Output: none
# Return: scalar, status
#
sub updusr_get_tablefile {
    my ($inst_spec,$flags) = @_;

    $inst_spec->{table_dir} = 
		"$inst_spec->{database}/$inst_spec->{product}";

    if ($flags =~ m/separate_tables/) {
        $inst_spec->{table_file} = "$inst_spec->{version}$inst_spec->{flavor}.table";
    } else {
        $inst_spec->{table_file} = "$inst_spec->{version}.table";
    }

    $inst_spec->{unwind_table_dir} = $inst_spec->{table_dir};
    $inst_spec->{unwind_table_file} = $inst_spec->{table_file};

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_get_upsdir( \%inst_spec)
#
# get ups subdirectory of product instance
#
# Input : hash, inst_spec	
# Output: none
# Return: scalar, status
#
sub updusr_get_upsdir {
    my ($inst_spec,$flags) = @_;

    # for most installations, the ups directory falls underneath
    # the product root.
    # ----------------------------------------------------------
    $inst_spec->{ups_dir} = "ups";
    $inst_spec->{unwind_ups_dir} = "$inst_spec->{unwind_prod_dir}/ups";

    return 1;
}

#
#-------------------------------------------------------------------------
# status = updusr_unwind_all ( \%inst_spec)
#
# Only used on upd server machines
#
# Input : hash, inst_spec	
#         flags - -o option from addproduct command line
# Output: none
# Return: boolean
#
# return true if product should be completely unwound, 0 if partialy
#
sub updusr_unwind_all {
    return 0;
}

1;

__END__

