##########################################################################
#
# FILE:
#       updusr.pm
# 
# DESCRIPTION: 
#       This module contain routines that are user callouts to get various
#       pieces of information.
#       (for more info, see pod at the end).
#
# MODIFICATIONS:
#	1998-03-20 mengel@fnal.gov	Initial "new kits" version
#
##########################################################################
package updusr;

use upderr;
use updups;
use updgbl;
use upduti;
use upderr;
use strict;
use integer;
use POSIX;
use File::Path;
use Exporter();
use vars qw( @ISA @EXPORT );
@ISA = qw( Exporter );

# export this !
@EXPORT = qw (
 	updusr_get_db
 	updusr_get_proddir
	updusr_predeclare
	updusr_postdeclare
	updusr_get_tablefile
	updusr_get_upsdir
	updusr_unwind_all
	);

##########################################################################
# Exported functions
#

# Location of kits area
#
$updusr::kitsarea="/ftp";


#-------------------------------------------------------------------------
# status = updusr_get_db ( \%inst_spec, \%top_inst_spec)
#
# Gets and stores (into my_instance_spec) the database into which the
# product should be declared.
#
# Input : hash, inst_spec	instance to get db for
#         hash, top_inst_spec	for overlaid products, this is the main
#                               product, otherwise it doesnt matter.
#         scalar, dist_db	distribution database
# Output: none
# Return: scalar, status
#
sub updusr_get_db {
my ($inst_spec, $top_inst_spec, $flags) = @_;
my @products;				# $PRODUCTS
my $db;					# single db
my $dist_node = $g_flags{opt_h};	# distribution node in case we need it.
					# this should be a hash!

$inst_spec->{database} = $updusr::kitsarea . 
			"/upsdb";

return 1;
}


#-------------------------------------------------------------------------
# status = updusr_get_proddir( \%real_inst_spec, \%top_inst_spec) 
#
# Gets and stores (into my_instance_spec) the directory into which the
# product files should be unwound and declared (these might be separated).
#
# Input : hash, real_inst_spec	instance to get dir for
#         hash, top_inst_spec	for overlaid products, this is the main
#                               product, otherwise it doesnt matter.
# Output: none
# Return: scalar, status
#
sub updusr_get_proddir {
    my ($real_inst_spec, $top_inst_spec, $flags) = @_;
    my $inst_spec;			# instance spec used
    my $dist_node = $g_flags{opt_h};	# distribution node in case we need it

    $inst_spec = $real_inst_spec;

    $real_inst_spec->{unwind_prod_dir} = $updusr::kitsarea .
					"/products/" . 
					$inst_spec->{product} . 
					"/" .
					$inst_spec->{version} .
					"/" .
					$inst_spec->{flavor};

    $real_inst_spec->{prod_dir} = $real_inst_spec->{unwind_prod_dir};

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_get_tablefile ( \%inst_spec)
#
# get table file location of product instance
#
# Input : hash, inst_spec	
# Output: none
# Return: scalar, status
#
sub updusr_get_tablefile {
    my ($inst_spec,$flags) = @_;

    $inst_spec->{table_dir} = "$inst_spec->{database}/$inst_spec->{product}";
    if ($flags =~ m/separate_tables/) {
        $inst_spec->{table_file} = "$inst_spec->{version}$inst_spec->{flavor}$inst_spec->{qualifiers}.table";
    } else {
        $inst_spec->{table_file} = "$inst_spec->{version}.table";
    }

    $inst_spec->{unwind_table_dir} = $inst_spec->{table_dir};
    $inst_spec->{unwind_table_file} = $inst_spec->{table_file};

    return 1;
}


#-------------------------------------------------------------------------
# status = updusr_get_upsdir( \%inst_spec)
#
# get ups subdirectory of product instance
#
# Input : hash, inst_spec	
# Output: none
# Return: scalar, status
#
sub updusr_get_upsdir {
    my ($inst_spec,$flags) = @_;

    # for most installations, the ups directory falls underneath
    # the product root.
    # ----------------------------------------------------------
    $inst_spec->{ups_dir} = "ups";
    $inst_spec->{unwind_ups_dir} = "$inst_spec->{unwind_prod_dir}/ups";

    return 1;
}

#
#-------------------------------------------------------------------------
# status = updusr_predeclare ( \%inst_spec)
#
# Only used on upd server machines
#
# Input : hash, inst_spec	
#         flags - -o option from addproduct command line
# Output: none
# Return: boolean
#
# set permissions, add symbolic links, etc. as needed
#
sub updusr_predeclare {
    my ($inst_spec, $flags) = @_;
    my $oldflav;			# trimmed flavor name
    my ($oldpath, $olddir);		# where old kits would put it
    my ($ftfile, $ftdir);		# where fermitools wants it
    my ($filemode, $dirmode,$group);	# file and directory modes
    my (@files, @dirs);			# files/dirs on which to set mode.
    my $suffix; 
    my $oldfile;

    upderr_log(3, "updusr_predecare( $inst_spec, $flags )\n");

    $suffix = (upduti_split_archive_file("ftp://node/$inst_spec->{archive_file}"))[5];
    $suffix = ".$suffix";

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   
    # make backward-compatability path/links
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   

    # figure out what the old directory tree would have been

    $oldflav = 	$inst_spec->{flavor};
    $oldflav =~ s/\+.*//;
    $oldflav =~ s/NULL/GENERIC_UNIX/;

    $olddir =  $updusr::kitsarea . 
		"/KITS/" .
		$oldflav .
		"/" .
		$inst_spec->{product} . 
		"/" .
		$inst_spec->{version};
    $oldfile = $inst_spec->{product} . 
		$inst_spec->{flavor} . 
		$inst_spec->{version} .
		$suffix;

    # make the directory if neccesary

    if (! -d $olddir) {
	unless (eval {mkpath($olddir);} ) {
	    upderr_log(0,"cannot make directory $olddir $!");
	    return 0;
	}
    }

    # make the symlink

    chdir($olddir);
    symlink($inst_spec->{prod_dir} . $suffix , $oldfile );

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   
    # do proprietary product stuff, FermiTools stuff
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   

    if ( $flags =~ /proprietary/ ) {

	# probably should do something else...

	$filemode = 0640;
        $dirmode = 0750;
	$group = $inst_spec->{product};
	$group =~ tr/[a-z]/[A-Z]/;

    } elsif ( $flags =~ /fermitools/ ) {

	# figure out where fermitools wants it
	$ftdir = $updusr::kitsarea .
		"/pub/" . 
		$inst_spec->{product} . "/" .
		$inst_spec->{version};
	$ftfile = $oldfile;

	# make the directory if neccesary

	if (! -d $ftdir) {
	    unless (eval {mkpath($ftdir);} ) {
		upd_errlog(0,"cannot make directory $ftdir $!");
		return 0;
	    }
	}

	# make the symlinks
	chdir($ftdir);
	symlink( $inst_spec->{archive_file}, $ftfile );
	symlink( "$inst_spec->{prod_dir}/README" , "README" );

	# pick the file mode well set

	$filemode = 0644;
        $dirmode = 0755;
	$group = 'upd';

    } else {

	# pick the file mode well set

	$filemode = 0640;
        $dirmode = 0750;
	$group = 'upd';
    }
    $group = (POSIX::getgrnam($group))[2];

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   
    # set file permissions
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   
    chmod( $filemode, $inst_spec->{archive_file});
    eval {chown( $<, $group, $inst_spec->{archive_file});} || 
	upderr_log(0,"chown $<.$group failed $!\n");

    chdir( $inst_spec->{prod_dir} );
    @files = split("\n", `find . -type f -print`);
    @dirs = split("\n", `find . -type d -print`);

    upderr_log(4,"file list is:\n" . join('\n', @files) . "\n");
    upderr_log(4,"dir list is:\n" . join('\n', @dirs) . "\n");
    upderr_log(4,"group id $group, file $filemode dir $dirmode\n");

    eval { chmod( $filemode, @files ); };
    eval { chmod( $dirmode, @dirs ); };
    eval { chown( $<, $group, @files , @dirs ); } ||
	upderr_log(0,"chown $<.$group failed $!\n");

    return 1;
}

sub updusr_postdeclare {
    my ($inst_spec, $flags) = @_;
    return 1;
}

#
#-------------------------------------------------------------------------
# status = updusr_unwind_all ( \%inst_spec)
#
# Only used on upd server machines
#
# Input : hash, inst_spec	
#         flags - -o option from addproduct command line
# Output: none
# Return: boolean
#
# return true if product should be completely unwound, 0 if partialy
#
sub updusr_unwind_all {
    my ($inst_spec, $flags) = @_;

    upderr_log(3, "updusr_unwind_all( $inst_spec, $flags )\n");
    if ($inst_spec->{flavor} =~ /WIN|NT/) {
        return 1;
    } else {
	return 0;
    }
}

1;

__END__

