##########################################################################
# $Id: default_updusr.pm,v 1.2 1998/07/31 17:50:18 mengel Exp $
#
# DESCRIPTION: 
#       This module contain routines that are user callouts to get various
#       pieces of information. Users are required to edit this file
#	and put in in $UPD_USERCODE or their home directory. Please
#	look at the comments for each subroutine to determine the 
#	pieces of information that need to be returned.
#
#	This file just provides examples, do NOT use as is. You MUST 
#	read and understand each function and modify it to conform to 
#	the configuration for your system. 
#
# MODIFICATIONS:
#
##########################################################################
package updusr;

use FileHandle;
use upderr;
use updups;
use updgbl;
use upduti;
use strict;
use integer;
use Exporter();
use vars qw( @ISA @EXPORT  
	     $updusr_table );
@ISA = qw( Exporter );

# export this !
@EXPORT = qw (
 	updusr_get_db
 	updusr_get_proddir
	updusr_predeclare
	updusr_postdeclare
	updusr_get_tablefile
	updusr_get_upsdir
	);

##########################################################################

# Implement the usual methods by looking up the answers in the
# table data structure.

my $updusr_table;

sub updusr_get_db {
    my ($inst_spec, $top_inst_spec,$flags) = @_;

    return get_from_table($inst_spec, $top_inst_spec, $flags, 'database');
}

sub updusr_get_proddir {
    my ($inst_spec, $top_inst_spec,$flags) = @_;

    return get_from_table($inst_spec, $top_inst_spec, $flags, 
				'unwind_prod_dir', 'prod_dir',
				'unwind_archive_file' ,'archive_file' );
}

sub updusr_get_tablefile {
    my ($inst_spec, $flags) = @_;

    return get_from_table($inst_spec, $inst_spec, $flags, 
				'unwind_table_dir',
				'table_dir', 
				'table_file' );
}

sub updusr_get_upsdir {
    my ($inst_spec,$flags) = @_;
    return get_from_table($inst_spec, $inst_spec, $flags, 
				'unwind_ups_dir',
				'ups_dir' );
}

sub updusr_predeclare {
    my ($inst_spec,$flags) = @_;

    return get_from_table($inst_spec, $inst_spec, $flags, 'predeclare');
}

sub updusr_postdeclare {
    my ($inst_spec,$flags) = @_;

    return  get_from_table($inst_spec, $inst_spec, $flags, 'postdeclare');
}

sub updusr_unwind_all {
    my ($inst_spec, $flags) = @_;
    return 0;
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
#
# parse the table, setting $updusr_table...
#

sub updusr_slurp_table {
     my ($inst_spec) = @_;
     my $fh = new FileHandle;
     my ($slot, $item, $part, $val, $filename);
     my ($before);
     my $syntaxerrors = 0;

     upderr_log(1,"Entering updusr_slurp_table\n");


     $updusr_table = [];

     #backward compatibility hack
     if ( -f "$::ENV{UPD_USERCODE_DIR}/config.upd" ) {
         $filename = "$::ENV{UPD_USERCODE_DIR}/config.upd";
     }

     if ( -f "$::ENV{UPD_USERCODE_DIR}/updconfig") {
          $filename = "$::ENV{UPD_USERCODE_DIR}/updconfig";
     }

     if ( !open( $fh, "<$filename" ) ) {
         upderr_log(0, "Unable to open '$filename'.\n");
	 return 0;
     }

     while ( <$fh> ) {
        upderr_log(3,"read: $_\n");

	#
	# fixup variable names with fixvars upon reading each line
        #
        $before = $_;
	s/([\\]?\$)(\{[A-Z_]+\})/$1.fixvars($1,$2)/ego;
	s/([A-Z_]+)(\s+=)/fixvars("", $1).$2/ego;
	s|\~[^"]*|(upduti_expandtildes($&))[0]|ego;

        upderr_log(5,"before: $before");
        upderr_log(5,"after: $_");

	if ( /\A\s*\Z/ || /\A\s*\#/  ) {

	    # comment, skip it...

	} elsif ( /group/i ) {

	    # start a new slot

            $slot = {};
            $slot->{matches} = {};
            $slot->{values} = {};
	    $slot->{predeclare} = "";
	    $slot->{postdeclare} = "";
            $part = 'matches';
            upderr_log(4,"starting matches..\n");

	} elsif ( /end/i ) {

	    # push the new slot on the table array

	    push(@$updusr_table,$slot);
            upderr_log(4,"adding slot $slot, list is @$updusr_table ..\n");
            upderr_log(4,"slot holds " . join(',',keys(%$slot)) . "\n");
            upderr_log(4,"slot values holds " . join(',',keys(%{$slot->{values}})) . "\n");

	} elsif ( /common/i ) {

	    # we'll now be setting values

	    $part = 'values';
            upderr_log(4,"starting values..\n");
        } elsif ( m/file\s*=\s*updconfig/i ) {

	    # okay, thats us...

        } elsif ( m/action\s*=\s*predeclare/i ) {

	    # we'll now be setting predeclare actions

	    $part = 'predeclare';
            upderr_log(4,"starting predeclare..\n");

        } elsif ( m/action\s*=\s*postdeclare/i ) {

	    # we'll now be setting postdeclare actions

	    $part = 'postdeclare';
            upderr_log(4,"starting postdeclare..\n");

	} elsif ( (m/(\S+)\s*=\s*(\S+)/ && $part eq 'matches') || (m/(\S+)\s*=\s*("\S+")/ && $part eq 'values')) {
	    ($item = $1) =~ tr{A-Z}{a-z};
	    $val = $2;
	    if ($part eq 'matches') {
		$val =~ s/\W/\\$&/go;
		$val =~ s/\AANY\Z/.*/;
		$val =~s/\\:/|/go;
		if (defined $slot->{$part}->{$item}) {
		   $val = $slot->{$part}->{$item} . '|' . $val;
		}
	    }
	    $slot->{$part}->{$item} = $val;
            upderr_log(4,"set $part $item to $val..\n");

	} elsif ( m/execute\s*\(\s*(.*),\s*NO_UPS_ENV\s*\)/i && ($part eq 'predeclare' || $part eq 'postdeclare')) {
	    $slot->{$part} .= "!system($1);" ;
            upderr_log(4,"added system($1) to $part..\n");

        } else {
	    upderr_log(0,"Syntax error in $filename line " .  $fh->input_line_number . "\n");
	    upderr_log(0,"line was: $before");
	    $syntaxerrors++;

	}
     }
     close($fh);
     return $syntaxerrors == 0;
}

sub get_from_table {
    my ($inst_spec, $top_inst_spec,$flags, @fields) = @_;
    my ($found, $slot, $item, $varlist);
    my ($product, $version, $flavor, $database, $qualifiers, 
	$dist_database, $dist_prod_dir, $dist_table_dir,
	$prod_dir, $unwind_prod_dir, 
	$ups_dir, $unwind_ups_dir,
	$table_dir, $unwind_table_dir,
	$table_file, $base_flavor ,
        $archive_file, $dist_archive_file, $unwind_archive_file);
    my %okvars;
    my $p;
    my $opt;

    $inst_spec->{"dist_node"} = $g_flags{opt_h};

    ($::ENV{SUFFIX} = ${dist_archive_file}) =~ s|.*(\..*)|$1|;
    %okvars  = (product=> 1, version=> 1, flavor=> 1, database=> 1, qualifiers=> 1, 
		dist_database=> 1, dist_prod_dir=> 1, dist_table_dir=> 1,
		prod_dir=> 1, unwind_prod_dir=> 1, 
		ups_dir=> 1, unwind_ups_dir=> 1,
		table_dir=> 1, unwind_table_dir=> 1,
		table_file=> 1, base_flavor => 1,
		archive_file=> 1, dist_archive_file => 1, 
		unwind_archive_file=>1 );

    upderr_log(1,"Entering get_from_table\n");

    if (defined $inst_spec->{database} && $inst_spec->{database} ne '' ) {
	$opt = "-z $top_inst_spec->{database}";
    } else {
        $opt = "";
    }
    # dig PROD_DIR_PREFIX out of that database...

    $p = `ups list ${opt} -K prod_dir_prefix 2>/dev/null`;
    $p =~ s/\s.*//so;
    upderr_log(2,"first list got |$p|\n");

    if ( $p eq '' || $p eq '""' ) {
	# XXX this is untill 'ups list -K prod_dir_prefix' with
	# XXX no other options works.

	$p = `ups list -a ${opt} -K prod_dir_prefix ups 2>/dev/null |  head -1`;
	$p =~ s/\s.*//so;
        upderr_log(2,"second list got $p\n");
	if ( $p eq '""' || $p eq '' ) {
	   $p = `ups list -a ${opt} -K prod_dir_prefix 2>/dev/null | head -1`;
	   $p =~ s/\s.*//so;
           upderr_log(2,"third list got $p\n");
	}
    }
    $p =~ s/"([^"]*)".*/\1/s;

    $::ENV{PROD_DIR_PREFIX} = $p;
    upderr_log(2,"Got PROD_DIR_PREFIX of $::ENV{PROD_DIR_PREFIX}\n");

    if ( ! defined $updusr_table ) { updusr_slurp_table() || return 0; }

    if ( !defined $inst_spec->{options} ) {
	$inst_spec->{options} = $g_flags{opt_O};
    }

    # import inst_spec keys as local variables

    $varlist = '';
    foreach $item (keys %{$top_inst_spec}) {
         if ($okvars{$item}) {
             $varlist .= "\$$item = '$top_inst_spec->{$item}'; ";
         }
    }
    upderr_log(3, "eval varlist is $varlist\n");

    if ( eval $varlist ) {
 	upderr_log(0, $@ );
    }

    # provide base flavor for folks who want it.
    $base_flavor = $flavor;
    $base_flavor =~ s/\+.*//;
    
    slotloop: 
    foreach $slot (@$updusr_table) {

	upderr_log(3,"trying slot $slot ..\n");
        # make sure all restrictions match
        $found = 1;
	foreach $item (keys (%{$slot->{matches}})) {
	    if ( ! ($inst_spec->{$item} =~ m/\A($slot->{matches}->{$item})\Z/) ) {
                upderr_log(3,"failed to match on $item \n");
                upderr_log(3,"item:" . $inst_spec->{$item}.   "\n");
                upderr_log(3,"pattern:" . $slot->{matches}->{$item} . "\n");
		$found = 0;
	    }
	}

	if ($found) {
	    # set all the fields for which we were queried
	    foreach $item (@fields) {
                upderr_log(3,"setting inst_spec-> $item to $slot->{values}->{$item}\n");
                if ( defined $slot->{values}->{$item}) {

		    upderr_log(3, "\$inst_spec->{$item} = $slot->{values}->{$item};\n");
		    if ( ! eval "\$inst_spec->{$item} = $slot->{values}->{$item};" ){
			upderr_log(0,$@);
		    }
		    if ( ! eval "\$$item = $slot->{values}->{$item};" ) {
			upderr_log(0,$@);
		    }
		    upderr_log(3,"got: $inst_spec->{$item} \n");
               } elsif (defined $slot->{$item} && $slot->{$item} ne '') {
	            upderr_log(3, "action is: $slot->{$item}\n");
		    if (eval $slot->{$item}) {
			return 1;
		    } else {
			upderr_log(0, "eval of action $slot->{$item}: $@\n" );
			return 0;
		    }
               } elsif (defined $slot->{$item} ) {
                    return 1;
               }
	    }
	    foreach $item (@fields) {
                upderr_log(3,"setting inst_spec-> $item to $slot->{values}->{$item}\n");
		upderr_log(3, "\$inst_spec->{$item} = $slot->{values}->{$item};\n");
		eval "\$inst_spec->{$item} = $slot->{values}->{$item};";
		eval "\$$item = $slot->{values}->{$item};";
                upderr_log(3,"got: $inst_spec->{$item} \n");
	    }
	    return 1;
	}
    }

    return 0;
}

#
# turn variabe names in the table file into variable names we use
# internally
#
sub fixvars {
    my ($pre,$string) = @_;
    my ($l, $r);
    local($_);
    
    $_ = $string;
    if ($pre eq "\\\$" || $string eq "ACTION" ) {
	return $_;
    }

    upderr_log(5,"fixvars: $_\n");
    if ( s/\{(.*)\}/\1/ ) { $l = "{"; $r = "}"; }
    if ( /\A(_.*)/ )		{ $_ = $1; tr[A-Z][a-z]; return $l.$_.$r; }
    if ( /UPS_THIS_DB/ )      	{ $_ = 'database'; return $l.$_.$r; }
    if ( /UPS_PROD_NAME/ )      { $_ = 'product'; return $l.$_.$r; }
    if ( /UPS_([A-Z]+_DIR)/ )   { $_ = $1; tr[A-Z][a-z]; return $l.$_.$r; }
    if ( /UPS_([A-Z]+_FILE)/ )  { $_ = $1; tr[A-Z][a-z]; return $l.$_.$r; }
    if ( /UPS_PROD_([A-Z]+)/ )  { $_ = $1; tr[A-Z][a-z]; return $l.$_.$r; }
    if ( /(UNWIND_[A-Z]+_DIR)/ ){ $_ = $1; tr[A-Z][a-z]; return $l.$_.$r; }
    if ( /(UNWIND_[A-Z]+_FILE)/ ){ $_ = $1; tr[A-Z][a-z]; return $l.$_.$r; }
    if ( /UPS_(BASE_FLAVOR)/ )  { $_ = $1; tr[A-Z][a-z]; return $l.$_.$r; }

    # others are just environment variables
    s/.*/::ENV{$&}/;
    return $_;
}

sub moduletest {
    my $inst_spec = { 	'product' => 'perl',
			'version' => 'v5_004',
			'flavor' => 'SunOS+5', 
			'dist_database' => '/ftp/upsdb',
		};
			
    $g_flags{opt_h} = "fnkits.fnal.gov";

    upduti_print_instance_spec(0, $inst_spec);
    upderr_get(); upderr_clear();

    updusr_get_db($inst_spec,$inst_spec, "");
    upderr_get(); upderr_clear();

    updusr_get_proddir($inst_spec,$inst_spec, "");
    upderr_get(); upderr_clear();

    updusr_get_tablefile ($inst_spec, "");
    upderr_get(); upderr_clear();

    updusr_get_upsdir($inst_spec, "");
    upderr_get(); upderr_clear();

    updusr_predeclare($inst_spec, "");
    upderr_get(); upderr_clear();

    updusr_postdeclare($inst_spec, "");
    upderr_get(); upderr_clear();

    upduti_print_instance_spec(0, $inst_spec);
    upderr_get(); upderr_clear();

    $inst_spec = { 	'product' => 'perl',
			'version' => 'v5_004',
			'flavor' => 'IRIX+5', 
                        'dist_database' => '/tmp/bogus',
			};
			
    upduti_print_instance_spec(0, $inst_spec);
    upderr_get(); upderr_clear();

    updusr_get_db($inst_spec,$inst_spec, "");
    upderr_get(); upderr_clear();

    upduti_print_instance_spec(0, $inst_spec);
    upderr_get(); upderr_clear();
}

if ( $0 =~ /(.*)updusr(.*)\.pm/ ) {
    $::ENV{UPD_USERCODE_DIR} = '.'; 
    upderr_setverbose(5);
    moduletest
}

1;
__END__
