package updmig;

use Cwd;
use upduti;
use updxfr;
use updstb;
use updups;
use updcmd;
use updgbl;
use upderr;
use updstb;
use strict;
use integer;
use File::Path;
use File::Copy;
use Exporter();

use vars qw( @ISA @EXPORT);
@ISA = qw(Exporter);

# export this !
@EXPORT = qw (
              updmig_migrate
              updmig_redotables
              );

sub updmig_migrate {
    my ($inst_spec) = @_;
    my ($proto,$host,$port,$dir,$file,$arch); # fields split from archive_file URL
    my ($flavor,$product,$version,$cust,$qualifiers);
    my %prop_info;
    my $status;
    my ($proplist,$isprop,$saidprop);
    my $local;
    my $live;
    my $tmploc;
 
    $live = ($g_flags{opt_O} =~ m/live/i);
    upderr_log(2," live flag is $live\n" );

    if ( !$live &&  !defined($inst_spec->{archive_file})) {
	upderr_log(0,"-T option is mandatory.\n");
	return 0;
    }


#---------------------------------------------------------------------
# we may need this at some point, but for now, we'll just open it up.
#
#    if ( $g_flags{opt_O} =~ m/proprietary/ ) {
#	my $junk;
#	$status = updui_prompt_propriety ($inst_spec, \%prop_info, \$junk);
#    }

    # default to stuff from archive file if needed
    if ( !$live &&
	(!defined($inst_spec->{flavor}) || $inst_spec->{flavor} eq '' ||
 	    !defined($inst_spec->{product}) || $inst_spec->{product} eq '' ||
	    !defined($inst_spec->{version}) || $inst_spec->{version} eq '' )) {

	upderr_log(2,"setting product flavor, etc. from tarfile name\n");

	($product,$version,$flavor,$qualifiers) = 
	  ($inst_spec->{archive_file} =~ 
		m{\A.*/([^/]*)/([^/]*)/\1\2([^+]*\+?[^+]*)(\+.*)?\.(\w+)\Z});
		
	$qualifiers = '' if (!defined($qualifiers));
	$qualifiers =~ s/\+/:/go;
	$qualifiers =~ s/^://;

	$inst_spec->{flavor}     = $flavor;
	$inst_spec->{product}    = $product;
	$inst_spec->{version}    = $version;
	$inst_spec->{qualifiers} = $qualifiers;
    }

    # get local db
    return 0 unless (&updstb_get_db($inst_spec,$inst_spec));

    unless ($live) {
        # get list of proprietary products from local db
	$local = upduti_parse_url("localhost");
	$proplist = updcmd_get_proprietarylist($local, $inst_spec->{database});
    }

    my $gotalready;
    updups_exist($local->{string}, $inst_spec, \$gotalready);

    if ($gotalready && $g_flags{opt_O} =~ m/date-([-\d]+)/) {
        my $remotedate = $1;
        my $localdate;
        my $savechain;
	my $havechain = 0;

	if (defined $inst_spec->{chain}) {
	   $havechain = 1;
           $savechain = $inst_spec->{chain};
	}
	delete $inst_spec->{declared};
	delete $inst_spec->{modified};
	delete $inst_spec->{chain};
        
        updups_find($local->{string}, $inst_spec, 'declared', 'modified');

	if ($havechain) {
            $inst_spec->{chain} = $savechain;
	}

	$gotalready = 0;
        foreach $localdate ( split(':', $inst_spec->{declared}),split(':',$inst_spec->{modified})) {
	    upderr_log(4,"Checking $localdate against $remotedate");
            if ($localdate ne '' && $localdate gt $remotedate) {
		$gotalready = 1;
	    }
	}
        if (!$gotalready) {
	    print "Need to reget $inst_spec->{product}, updated $remotedate\n";
	}
    }

    if ($gotalready && ! ($g_flags{opt_O} =~ m/force/ )) {
        print "Already have $inst_spec->{product} -f $inst_spec->{flavor} $inst_spec->{version}...\n";
        return 1;
    } else {
	my $action = $live ? "Cloning" : "Fetching";
        print "$action $inst_spec->{product} -f $inst_spec->{flavor} $inst_spec->{version}...\n";
    }

    unless ($live) {
	$isprop = $saidprop = 0;
	$isprop = 1 if ($proplist =~ m/:$inst_spec->{product}:/);
	$saidprop = 1 if ($g_flags{opt_O} =~ m/proprietary/ );
	upderr_log(3, "isprop == $isprop saidprop == $saidprop\n");

	if ($isprop != $saidprop) {
	    upderr_log(0,"product $inst_spec->{product} must be migrated with -O proprietary\n");
		    upderr_get(); upderr_clear();
		return 0;
	}

	upderr_log(3,"updmig_migrate: archive_file is $inst_spec->{archive_file}\n");
	($proto,$host,$port,$dir,$file,$arch) =
		upduti_split_archive_file($inst_spec->{archive_file});

	upderr_get(); upderr_clear();

	$tmploc = "/tmp/mig$$.$arch";

	if ( $host eq "localhost" ) {
	    $inst_spec->{archive_file} = "$dir/$file.$arch";
	} else {
	    updxfr_file($host, $dir, "${file}.${arch}", $tmploc, \%prop_info);
	    upderr_get(); upderr_clear();
	    $inst_spec->{archive_file} = $tmploc;
	}

	upderr_log(3,"before move: archive_file is $inst_spec->{archive_file}\n");

	# put it where addproduct would
	return 0 unless
	    updcmd_move_archive_file( $inst_spec );
    }

    return 0 unless
	(&updstb_get_tablefile($inst_spec));
    
    upderr_log(3,"after move: archive_file is $inst_spec->{archive_file}\n");
    upderr_log(3,"            table_file is $inst_spec->{table_file}\n");
    upderr_log(3,"            unwind_table_dir  is $inst_spec->{unwind_table_dir}\n");

    # make a table file
    if (eval {chdir($inst_spec->{ups_dir});} ) {
        updmig_maketable( $inst_spec );
    }  elsif (eval {chdir($inst_spec->{prod_dir} . '/' . $inst_spec->{ups_dir});} ) {
        updmig_maketable( $inst_spec );
    } else {
	print "No ups dir from which to make a table file\n";
        delete $inst_spec->{unwind_table_dir};
        delete $inst_spec->{table_dir};
        delete $inst_spec->{table_file};
    }

    # fix the archive_location to a full URL

    unless ($live) {
        $host = $g_flags{opt_h};
        $inst_spec->{archive_file} =~ s|.*|ftp://$host$&|;
        unlink($tmploc);
    }

    # declare the product
    updstb_predeclare($inst_spec);
    updups_declare( 0, $inst_spec, {}, "-C" );
    updups_touch( 0, $inst_spec, {});
    updstb_postdeclare($inst_spec);
}

sub updmig_redotables {

    my ($inst_spec) = @_;
    upderr_log(0,"Redoing tables for $inst_spec->{product} $inst_spec->{flavor} $inst_spec->{version}\n");
    # get list of proprietary products from local db
    return 0 unless (&updstb_get_db($inst_spec,$inst_spec));
    delete $inst_spec->{prod_dir};
    delete $inst_spec->{ups_dir};
    delete $inst_spec->{unwind_table_dir};
    delete $inst_spec->{table_dir};
    delete $inst_spec->{table_file};
    updups_find(0,$inst_spec,'prod_dir','ups_dir','table_dir','table_file');
    $inst_spec->{unwind_table_dir} = $inst_spec->{table_dir};

    if (!defined $inst_spec->{ups_dir} || $inst_spec->{ups_dir} eq '' ) {
       $inst_spec->{ups_dir} = 'ups';
    }
    if (eval {chdir($inst_spec->{prod_dir} . '/' . $inst_spec->{ups_dir});} ) {
        return updmig_maketable( $inst_spec );
    } else {
	print "No ups dir from which to make a table file\n";
    }
}

sub updmig_maketable {
    my ($inst_spec);
    my (@fields);
    my $reqtype;
    my $cust;
    my $prod;
    my $flavor;
    my $flags;
    my $vers;
    my @quals;
    my $quals;
    my %reqs;
    my ($action, $saw);
    my ($set, $qualbase);
    my $fullpath;
    my $localflavor;

    ($inst_spec) = @_;


    upderr_log(2,"entering updmig_maketable\n");
    $reqtype = '';
    $reqs{x} = [];
    $reqs{'xbuild:'} = [];
    @quals = ();

    if ( ! -d $inst_spec->{unwind_table_dir} ) {
	if (!eval{ mkpath( $inst_spec->{unwind_table_dir}); } ) {
	    upderr_log(0, "unable to make directory $inst_spec->{unwind_table_dir}\n");
	    return 0;
	}
    }

    $fullpath = "$inst_spec->{unwind_table_dir}/$inst_spec->{table_file}";

    if (-f $fullpath) {
        my $firstline; 
        open(TABLE, "<$fullpath");
        $firstline = <TABLE>;
        close(TABLE);
	return 1 unless ($firstline =~ m/remove this line/);
    }

    if (open(TABLEIN, "<$inst_spec->{product}.table") ||
        open(TABLEIN, "<action.table") ) {
	unless (open(TABLE, ">$fullpath" )) {
	    upderr_log(0,"unable to write table file $fullpath\n");
            close(TABLEIN);
	    return 0;
	}
        copy(\*TABLEIN,\*TABLE);
        close(TABLEIN);
        close(TABLE);
        return 1;
    }

    if (open(DECLDAT , "<declare.dat")) {
	$saw = 0;
	$localflavor = '';
	while ( <DECLDAT> ) {
	    @fields = split(/\s+/);
	    upderr_log(4, "got: " . join('|', @fields) . "\n");
	    if ( $fields[0] eq 'Flavor')    { 
                # trim to base flavor for later comparisons
		$localflavor = $fields[2]; 
		$localflavor =~ s/\+.*//;
            }
	    if ( $fields[1] eq 'USE') 	 { $saw = 1; $reqtype = 'x';   }
	    if ( $fields[1] eq 'BUILD')	 { $saw = 1; $reqtype = 'xbuild:'; }
	    if ( $fields[1] eq 'ORDER:') { $saw = 1; $prod = $fields[4]; }
	    if ( $fields[1] eq 'VERSION:')	{ 
		$saw = 1; 
		$vers = $fields[2]; 
		# check for chains!
                if ( $vers =~ m/current|test|development|new/ ) {
		    $vers = "-g " . $vers;
                }
	    }
	    if ( $fields[1] eq 'FLAGS:')	{ 
		$saw = 1; 
		$flags = join(' ', @fields[2..$#fields]); 
		if ($flags =~ s/-b//) {
		    push(@quals,'build');
		}
	    }
	    if ( $fields[1] eq 'FLAVOR:') 	{ 
		$saw = 1; 

		($flavor,$cust,$quals) = 
		    ($fields[2] =~ m/([^+]*)(\+[^+]*)?(\+.*)?/);

		$quals =~ s/^\+//;
		push(@quals,split('\+',$quals));

                # if its the same base flavor as our current product
                # let flavor matching do it.  Otherwise its some sort
		# of cross-flavor dependency, so keep it explicit

		if ($flavor eq $localflavor) {
		    # $flavor = '-f ${UPS_PROD_FLAVOR}';
		    $flavor = '';
		} else {
		    $flavor = "-f $flavor$cust"; 
		}
	    }
	    if ( $#fields == -1 && $saw ) {
                if ( $#quals > 0 ) {
		    $quals = "-q " . join(':', @quals);
                } else {
 		    $quals = "";
                }
		$action = "setupRequired($flavor $quals $flags $prod $vers )";
		upderr_log(3, "adding: $action\n");
		push( @{$reqs{$reqtype}} , $action);
		$saw = 0;
		@quals = ();
	    }
	}
	close DECLDAT;
    }

    unless (open(TABLE, ">$fullpath" )) {
	upderr_log(0,"unable to write table file $fullpath\n");
	return 0;
    }

    upderr_log(3,"writing $fullpath\n");

    print TABLE "# Auto generated -- remove this line to keep your changes\n";
    print TABLE "FILE=table\n";
    print TABLE "PRODUCT=$inst_spec->{product}\n";


    foreach $set ( 'x', 'xbuild:') {
	my @list;

        upderr_log(3,"list " . $reqs{$set} . " = " . join(':',@{$reqs{$set}}) . "\n");
	$reqs{setup} = $reqs{$set};

	$qualbase = $set;
	$qualbase=~ s/^x//;

	print TABLE "GROUP:\n";
	print TABLE " FLAVOR=ANY\n";
	print TABLE " QUALIFIERS=$qualbase$inst_spec->{qualifiers}\n";
	print TABLE "\n";
	print TABLE "COMMON:\n";
	foreach  $action ( "setup", "configure", "current", "tailor", "unconfigure", "uncurrent" ) {
	    if ($#{$reqs{$action}} == -1 && !-f ${action} && !-f "${action}.sh") {
		 next;
	    }
	    print TABLE "  ACTION=${action}\n";
	    print TABLE "    " . join("\n    ", @{$reqs{$action}}) . "\n"
		if ($#{$reqs{$action}} > -1);
	    if ( !-f "${action}.sh" && -f "${action}.csh" && ${action} eq 'setup' ) {
		print TABLE "    setupenv()\n";
		print TABLE "    proddir()\n";
	        print TABLE "    SourceOptional(\${UPS_UPS_DIR}/${action}.\${UPS_SHELL},UPS_ENV)\n";
	    } elsif ( -f "${action}.sh" && -f "${action}.csh" && ${action} eq 'setup' ) {
		print TABLE "    setupenv()\n";
		print TABLE "    proddir()\n";
	        print TABLE "    SourceRequired(\${UPS_UPS_DIR}/${action}.\${UPS_SHELL},UPS_ENV)\n";
	    } elsif ( -f ${action} && ${action} eq 'setup' ) {
		print TABLE "    setupenv()\n";
		print TABLE "    proddir()\n";
                print TABLE "    envSetIfNotSet(UPS_COMPAT,`echo .\${UPS_SHELL}|sed -e 's/.csh//'`)\n";
	        print TABLE "    SourceOptional(\${UPS_UPS_DIR}/${action}\$UPS_COMPAT,UPS_ENV)\n";
	    } elsif ( -f ${action} ) {
	        print TABLE "    proddir()\n";
		if ( $action eq "current" && -f "configure" ) {
	            print TABLE "    Execute(\${UPS_UPS_DIR}/configure,UPS_ENV)\n";
		}
	        print TABLE "    Execute(\${UPS_UPS_DIR}/${action},UPS_ENV)\n";
	        print TABLE "    unproddir()\n";
	    }
	}
	print TABLE "END:\n";
    }

    close TABLE;

    return 1;
}

if ( $0 =~ /updmig.pm\Z/ ) {

	$updmig::inst_spec = { 'product' => 'exmh', 'qualifiers' => '' };
	$updmig::inst_spec->{unwind_table_dir} = getcwd();
	$updmig::inst_spec->{table_file} = "$updmig::inst_spec->{product}.table";
	upderr_setverbose(4);
	updmig_maketable($updmig::inst_spec);
	upderr_get();
}

1;
