##########################################################################
#
# FILE:
#       uppmail.pm
# 
# DESCRIPTION: 
#       Functions to handle the upp mail messages.
#
# AUTHORS:
#       Marc Mengel
#       Lars Rasmussen
#       Margaret Votava
#       Don Walsh
#
#       Fermilab Computing Division
#       Batavia, Il 60510, U.S.A.                                              
#
# MODIFICATIONS:
#       dd-mmm-yyyy, NN, first
#
##########################################################################

package uppmail;

use upderr;

require Exporter;

@ISA = qw(Exporter);

##########################################################################
# Exported Functions
#
##########################################################################

@EXPORT = qw (	uppmail_log
		uppmail_get
	      	uppmail_send
	      	uppmail_clear
	      	);

##########################################################################
# Global and local variables
#
##########################################################################

my $MAILMSGS = '';		# store error messages here till printed

##########################################################################
# What to use
#
##########################################################################

use strict;
use integer;
use Cwd;

#-------------------------------------------------------------------------
# uppmail_log(scalar verbosity, $message)
#
# Log messages to a $MAILMSG for later use by the mailer.
#
# Input : scalar, $message
# Output: none
# Return: none
#

sub uppmail_log {
	my ( $msg ) = @_;
	
	upderr_log(4,"$msg");

	$MAILMSGS .= $msg;
	return;		
}

sub uppmail_get {
 	return $MAILMSGS;
}

#-------------------------------------------------------------------------
# $status = uppmail_send( \%mailInfo, \%file_header)
#
# Sends mail to the user.
#
#
# Input : 
#         hash, $mailInfo
#         hash, $file_header
# Output: none
# Return: status, 0 if any errors
#

sub uppmail_send {
	my ( $mailptr ) = @_;

	unless (open MAIL, "|/usr/lib/sendmail -t") {
		upderr_log (0, "Can't open Mail filehandle...\n");
		return 0;
	}

	print MAIL "To: $mailptr->{ 'to' }";
	print MAIL "From: $mailptr->{ 'from' }";
	
	if (defined $mailptr->{ 'cc' } ) {
		print MAIL "Cc: $mailptr->{ 'cc' }";
	}
	print MAIL "Subject: $mailptr->{ 'subject' }";
	print MAIL "\n";
	print MAIL "$mailptr->{ 'message' }\n";
	
	print MAIL "\n";
	print MAIL $MAILMSGS;
	close MAIL;
	return 1;
}

#-------------------------------------------------------------------------
# uppmail_clear()
#
# Sets the global $MAILMSG ''
# current level.
#
# Input : none
# Output: none
# Return: none
#
sub uppmail_clear {

	$MAILMSGS = '';
}

1;

__END__

=head1 NAME

upderr - blah blah

=head1 SYNOPSIS

<use it this way>

=head1 DESCRIPTION

<description>

=head1 BUGS

<bugs>

=cut
