##########################################################################
#
# FILE:
#       upputil.pm
# 
# DESCRIPTION: 
#       UPP 
#       (for more info, see pod at the end).
#
# AUTHORS:
#       Marc Mengel
#       Lars Rasmussen
#       Margaret Votava
#       Don Walsh
#
#       Fermilab Computing Division #       Batavia, IL 60510, U.S.A.                                              # # MODIFICATIONS:
#
##########################################################################

package upputil;

use upderr;

use File::Copy;
use FileHandle;
use strict;
use Exporter();
use vars qw( @ISA @EXPORT );

@EXPORT = qw (
		upp_readDataFile
		upp_writeDataFile
		);

@ISA = qw( Exporter );

my @Fields = ( 'product', 'version', 'declared');

#-------------------------------------------------------------------------
# $status = upp_readDataFile( $filename, \%a_inst, \%r_inst)
#
# Reads the upp data file containg the products and dates.
#
# Input : 
#         scalar, $filename
#         hash,  $a_inst
#         hash,  $r_inst
#
# Output: If the product is found it fills in the $r_inst
# Return: 1 - if product is found
#		: 2 - if product is not found
#		: 0 - if error
#

sub upp_readDataFile {
	my ( $filename, $a_inst, $r_inst ) = @_;

my $DATAFILE = new FileHandle;
my $i = 0;

	unless ( -e $filename ) {	# upp data file does not exist
		return 2;
	}

	unless (open $DATAFILE, $filename ) {
 		upderr_log( 0, "ERROR: Can't open file: $filename.\n");
		return 0;
	}

	while( <$DATAFILE> ) {
		if ((/^#/ ) or ( ! /\w/ )) { # skip comments and blank lines
			next;
		}
		chop $_;
		for $i (0 .. $#Fields ) {
			$r_inst->{ $Fields[$i] } = $_;
		}
		if ($a_inst->{'product'} eq $r_inst->{'product'} ) {
			close $DATAFILE;
			return 1;	# product found
		}
	}
	close $DATAFILE;
	return 2;			# Product not found
}
			
#-------------------------------------------------------------------------
# $status = upp_writeDataFile( $filename, \%a_inst)
#
# Appends the product instance to 'filename'.
#
# Input : 
#         scalar, $filename
#         hash,  $a_inst
# Output: none
# Return: status, 1 if successful else 0
#

sub upp_writeDataFile {
	my ( $filename, $a_inst ) = @_;
	
my $field = '';
my $DATAFILE = new FileHandle;
	
	unless (open $DATAFILE, ">>$filename" ) {
  		upderr_log( 0, "ERROR: Can't open file for writing: $filename\n");
		return 0;
	}
	foreach $field (@Fields) {
		print $DATAFILE "$a_inst->{$field}\n";
	}
	print $DATAFILE "#-------------------------\n";
	close $DATAFILE;
	return 1;
}

#-------------------------------------------------------------------------
# $status = print_hash( $fh, \%prodrec)
#
#	Print the contents of the hash.
#
# Input : 
#         hash, $hptr
# Output: none
# Return: 1
#

sub print_hash {
	my ( $hptr ) = @_;
	
my $elem;

	foreach $elem ( keys  %$hptr ) {
		print "$elem: $hptr->{$elem}\n";
	}
	return 1;
}
